﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using C1.Win.C1Chart;

//
// The last routine in this file contains code to identify
// the closest Chart TrendLine to the specified mouse point.
//

[assembly: AssemblyTitle("TrendLineSelection")]
[assembly: AssemblyCompany("ComponentOne, LLC")]
[assembly: AssemblyProduct("TrendLineSelection")]
[assembly: AssemblyCopyright("Copyright © ComponentOne, LLC 2011")]
[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyFileVersion("1.0.0.0")]

namespace TrendLineSelection
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
		}

		private void c1Chart1_Load(object sender, EventArgs e)
		{
			c1Chart1.Header.Text = "Move the mouse over the chart trendlines.";
            c1Chart1.Header.Visible = true;

			AddChartTrendLines(c1Chart1.ChartGroups.Group0.ChartData);
		}

		private void AddChartTrendLines(ChartData chartData)
		{
			for (int si = 0; si < chartData.SeriesList.Count; si++ )
			{
				ChartDataSeries cds = chartData.SeriesList[si];
				TrendLine tl = chartData.TrendsList.AddNewTrendLine();
				tl.SeriesIndex = si;
				tl.TrendLineType = TrendLineTypeEnum.Polynom;
				tl.RegressionOptions.NumTerms = 4;

				Color c = cds.LineStyle.Color;
				tl.LineStyle.Color = Color.FromArgb((c.R + 255) / 2, (c.G + 255) / 2, (c.B + 255) / 2);
				tl.LineStyle.Thickness = 3;
			}
		}

		int lastHighlightedTrendLineIndex = -1;
		private void processMouseOp(object sender, MouseEventArgs e)
		{
			ChartGroup cg = c1Chart1.ChartGroups.Group0;
			Rectangle brect = new Rectangle(c1Chart1.ChartArea.PlotArea.Location,
				c1Chart1.ChartArea.PlotArea.Size);
			int maxPixelDistance = 3;

			int tlIndex = FindNearestTrendline(cg, brect, e.Location, maxPixelDistance);

			if (tlIndex != lastHighlightedTrendLineIndex)
			{
				TrendLinesCollection tlc = cg.ChartData.TrendsList;
				int last = lastHighlightedTrendLineIndex;
				if (tlIndex >= 0 && tlIndex < tlc.Count)
				{
					TrendLine tl = tlc[tlIndex];
					tl.LineStyle.Thickness *= 2;
					lastHighlightedTrendLineIndex = tlIndex;
				}
				else
					lastHighlightedTrendLineIndex = -1;

				if (last >= 0 && last <= tlc.Count)
				{
					TrendLine tl = tlc[last];
					tl.LineStyle.Thickness /= 2;
				}

                // Show status label
                if (tlIndex < 0)
                    label1.Text = "Selection: none";
                else
                    label1.Text = "Selection: " + cg.ChartData.TrendsList[tlIndex].SeriesLabel +" TrendLine" ;
	
			}
		}

		private void c1Chart1_MouseMove(object sender, MouseEventArgs e)
		{
			processMouseOp(sender, e);
		}

		/// <summary>
		/// Given the nearest Mouse coordinate, finds the index of closest trendline less than the
		/// specified number of pixels distant.  An index of -1 indicates no trendline within the
		/// specified number of pixels.
		/// </summary>
		/// <param name="cg">The ChartGroup with the trendlines to consider.</param>
		/// <param name="brect">The bounding rectangle of the PlotArea.</param>
		/// <param name="p">Mouse coordinate.</param>
		/// <param name="maxPixelDistance">Largest distance in pixels to consider</param>
		/// <returns></returns>
		private int FindNearestTrendline(ChartGroup cg, Rectangle brect, Point p, int maxPixelDistance)
		{
			int tlIndex = -1;

			// if the point value lies within the chart data coordinate region
			// there could be a trendline near
			if (brect.Contains(p))
			{
				ChartData cd = cg.ChartData;
				TrendLinesCollection tlc = cg.ChartData.TrendsList;

				// find the minimum and maximum X pixel values within range
				int pixMinX = ((p.X - maxPixelDistance) >= brect.Left) ? (p.X - maxPixelDistance) : brect.Left;
				int pixMaxX = ((p.X + maxPixelDistance) <= brect.Right) ? (p.X + maxPixelDistance) : brect.Right;

				// minimum square is enough to identify the closest
				float maxDistSquared = maxPixelDistance * maxPixelDistance;
				float minDistSquared = float.MaxValue;

				for (int ti = 0; ti < tlc.Count; ti++)
				{
					TrendLine tl = tlc[ti];
					float xf = 0f, yf = 0f;

					// for each x pixel value, determine the trendline y pixel value
					for (int px = pixMinX; px <= pixMaxX; px++)
					{
						double xv = 0, yv = 0;
						if (cg.CoordToDataCoord(px, p.Y, ref xv, ref yv))
						{
							// get the trendline y value for the calculated x value
							yv = tl.GetY(xv);

							// now get the pixel x and y values
							if (cg.DataCoordToCoord(xv, yv, ref xf, ref yf))
							{
								// calculate and compare minimum distance squared values
								float minds = (xf - p.X) * (xf - p.X) + (yf - p.Y) * (yf - p.Y);
								if (minds < maxDistSquared && minds < minDistSquared)
								{
									// smallest min distance squared value so far and within range.
									tlIndex = ti;
									minDistSquared = minds;
									if (minds == 0f) break;		// will not get closer than zero
								}
							}
						}
					}
				}
			}
			return tlIndex;
		}
	}
}
